<?php

namespace Ems\Controllers;

use Exception\Messages;
use Service\LogService;
use Service\OAuth\OAuthCustomImpl;
use Zeedhi\Framework\Controller\Simple;
use Zeedhi\Framework\HTTP\Kernel;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;

use Ems\Services\MinhaHistoriaService;
use Ems\Util\Utils;

class MinhaHistoriaController {
    protected $minhaHistoriaService;

    protected $bloqDiaIgualSoli = ['3', '9', '21', '22', '27', '29'];

    public function __construct(MinhaHistoriaService $minhaHistoriaService) {
        $this->minhaHistoriaService = $minhaHistoriaService;
    }

    public function solicitaAlteracao(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();

            if ($row["NRTIPOSOLICITACAO"] == 16) {
                $nrSolicitacao = $this->minhaHistoriaService->solicitaAlteracaoComunica($row);
            } else {
                $nrSolicitacao = $this->minhaHistoriaService->solicitaAlteracao($row);
            }

            $dataset = [
                'success' => true,
                'nrsolicitacao' => $nrSolicitacao,
            ];

            $response->addDataSet(new DataSet('solicitaAlteracao', $dataset));
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getSolicitacaoFile(
        Request\Row $request,
        Response $response
    ) {
        try {
            $row = $request->getRow();

            $nrSolicitacao = (int) $row['NRSOLICITACAO'];

            $files = $this->minhaHistoriaService->
                getSolicitacaoFile($nrSolicitacao);

            $response->addDataSet(new DataSet('getSolicitacaoFile', $files));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function getSolicitacoesDia(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();

            $nrSoliAlte = false;
            $dsSoliAlte = '';
            $msg = null;
            
            if(in_array($row["NRTIPOSOLICITACAO"], $this->bloqDiaIgualSoli)) {
                $existeSolicitacao = $this->minhaHistoriaService->verificaBloqueioSolicitacao($row);
                if($existeSolicitacao && isset($existeSolicitacao[0]['NRSOLICITACAO'])) {
                    $nrSoliAlte = $existeSolicitacao[0]['NRSOLICITACAO'];
                    $dsSoliAlte = "uma solicitação";
                    $dsFimSoliAlte = "deste tipo para o dia selecionado. Favor selecionar outro dia";
                } else if($existeSolicitacao && isset($existeSolicitacao[0]['ALTERACAO'])) {
                    $nrSoliAlte = $existeSolicitacao[0]['ALTERACAO'];
                    $dsSoliAlte = "uma alteração";
                    $dsFimSoliAlte = "deste tipo para o dia selecionado. Favor selecionar outro dia";
                } else if($existeSolicitacao && isset($existeSolicitacao[0]['BENEFICIO'])) {
                    $nrSoliAlte = $existeSolicitacao[0]['BENEFICIO'];
                    $dsSoliAlte = "um benefício";
                    $dsFimSoliAlte = "deste tipo em ativo. Favor verificar a vigência";
                } else if($existeSolicitacao && isset($existeSolicitacao[0]['EVENTO'])) {
                    $nrSoliAlte = $existeSolicitacao[0]['EVENTO'];
                    $dsSoliAlte = "um evento relacionado";
                    $dsFimSoliAlte = "deste tipo em ativo. Favor verificar a vigência";
                }
            }
            
            if($nrSoliAlte) {
                $msg = 'Já existe ' . $dsSoliAlte . ' (N° '. $nrSoliAlte . ') ' . $dsFimSoliAlte;
            }
            
            $dataset = [
                'success' => true,
                'mensagem' => $msg,
            ];
            
            $response->addDataSet(new DataSet('getSolicitacoesDia', $dataset));
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }
    
    public function getNacionalidade(Request\Row $request, Response $response) {
        try {
            $dataset = $this->minhaHistoriaService->getNacionalidade();
            
            $response->addDataSet(new DataSet('getNacionalidade', $dataset));

        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }
    
    public function getPais(Request\Row $request, Response $response) {
        try {
            $dataset = $this->minhaHistoriaService->getPais();
            
            $response->addDataSet(new DataSet('getPais', $dataset));

        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getEstado(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            
            $cdPais = $row['CDPAIS'] ?? '0055';
            
            $dataset = $this->minhaHistoriaService->getEstado($cdPais);
            
            $response->addDataSet(new DataSet('getEstado', $dataset));

        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }
    
    public function getMunicipio(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            
            $cdpais = $row['CDPAIS'] ?? '0055';
            $cdestado = $row['SGESTADO'] ?? null;
            
            $dataset = $this->minhaHistoriaService->getMunicipio($cdpais, $cdestado);
            
            $response->addDataSet(new DataSet('getMunicipio', $dataset));

        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }
    
    public function getGrauinstrucao(Request\Row $request, Response $response) {
        try {
            $dataset = $this->minhaHistoriaService->getGrauinstrucao();
            
            $response->addDataSet(new DataSet('getGrauinstrucao', $dataset));

        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }
    
    public function getClasseEstrang(Request\Row $request, Response $response) {
        try {
            $dataset = $this->minhaHistoriaService->getClasseEstrang();
            
            $response->addDataSet(new DataSet('getClasseEstrang', $dataset));

        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }
    
    public function getConselhoRegional(Request\Row $request, Response $response) {
        try {
            $dataset = $this->minhaHistoriaService->getConselhoRegional();
            
            $response->addDataSet(new DataSet('getConselhoRegional', $dataset));

        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }
    
    public function getTpEndereco(Request\Row $request, Response $response) {
        try {
            $dataset = $this->minhaHistoriaService->getTpEndereco();
            
            $response->addDataSet(new DataSet('getTpEndereco', $dataset));

        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }
    
    public function getLogradouro(Request\Row $request, Response $response) {
        try {
            $dataset = $this->minhaHistoriaService->getLogradouro();
            
            $response->addDataSet(new DataSet('getLogradouro', $dataset));

        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }
    
    public function getBanco(Request\Row $request, Response $response) {
        try {
            $dataset = $this->minhaHistoriaService->getBanco();
            
            $response->addDataSet(new DataSet('getBanco', $dataset));

        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }
    
    public function getAgencia(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            $cdbanco = $row['CDBANCO'] ?? null;
            
            $dataset = $this->minhaHistoriaService->getAgencia($cdbanco);
            
            $response->addDataSet(new DataSet('getAgencia', $dataset));

        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }
    
    public function getTpContaCorrente(Request\Row $request, Response $response) {
        try {
            $dataset = $this->minhaHistoriaService->getTpContaCorrente();
            
            $response->addDataSet(new DataSet('getTpContaCorrente', $dataset));

        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }
}
