<?php

namespace Ems\Logic\ProcessamentoSolicitacao;

use Ems\Util\Utils;
use Util\{ DateUtil, NovoCodigo, Repositories };

class ProcessamentoSolicitacaoSalarioEscala extends ProcessamentoSolicitacao {
    
    public function efetivar() {
        $itens = $this->getItensAssociativeArray();
        
        $nrorg             = Utils::getParamters()->getNrorg();
        $nrvinculom        = $this->getSolicitacao()->getNrvinculom();
        $dtcompetencia     = DateUtil::getDataDeString(Utils::getParamters()->getDtmescomptencDiaCorrente(), DateUtil::FORMATO_BRASILEIRO,true);

        // SALÁRIO
        $nrtipoaltera   = $itens['NRTIPOALTERA']        ?? null;
        $idtpsalario    = $itens['IDTPSALARIO']         ?? null;
        $nrocorrencia   = $itens['NROCORRENCIA']        ?? null;
        $novoVlrSalario = $itens['VRSALARIO']           ?? $itens['NOVOVALORSALARIO'] ?? null;

        // ESCALA
        $nrturno        = $itens['NRTURNO']             ??  null;
        $nrmotivoescala = $itens['NRMOTIVOESCALA']      ??  null;
        $nrescalatrabm  = $itens['NRESCALATRABM']       ??  null;
        $dtiniescala    = isset($itens['DTINIESCALA']) ? DateUtil::getDataDeString(substr($itens['DTINIESCALA'],0,10), DateUtil::FORMATO_BRASILEIRO, true) : null;
        $dtcompetenciaString  = Utils::getParamters()->getDtmescomptencDiaCorrente();
        
        // OCUPAÇÃO
        $nrocupacaom        = $itens['NROCUPACAOM']     ?? $itens['NROCUPACAOM_OCU'] ?? null; 
        $nrmotivoocupa      = $itens['NRMOTIVOOCUPA']   ?? null; 
        $nrtpmodalidsal     = $itens['NRTPMODALIDSAL']  ?? null; 
        $nrtipoocupacao     = $itens['NRTIPOOCUPACAO']  ?? null;

        // Transferência/Remanejamento
        $nrtipotransfer = '1';
        $nrtpmovtransfm = $itens['NRTPMOVTRANSFM']  ?? null;
        $nrmotivotransf = $itens['NRMOTIVOTRANSF']  ?? null;
        $nrestrutlegal  = $itens['NRESTRUTLEGAL']  ?? null;
        $nrestrutgeren  = $itens['NRESTRUTGEREN']  ?? null;
        $dtinimoviment  = isset($itens['DTINIMOVIMENT']) ? DateUtil::getDataDeString(substr($itens['DTINIMOVIMENT'],0,10), DateUtil::FORMATO_BRASILEIRO, true) : null;
        $dtfimmoviment  = isset($itens['DTFIMMOVIMENT']) ? DateUtil::getDataDeString(substr($itens['DTFIMMOVIMENT'],0,10), DateUtil::FORMATO_BRASILEIRO, true) : null;

        $dsobsaltesalario  = $itens['DSOBSALTESALARIO'] ?? '';
        $observacao        = $this->getObservacaoComProtocolo($dsobsaltesalario);

        $camposObrigatorios = [
            'novoVlrSalario' => "O campo Salário é obrigatório!",
            'idtpsalario'    => "O campo Tipo de Salário é obrigatório!",
            'nrtipoaltera'   => "O campo Tipo de Alteração é obrigatório!",
            'nrescalatrabm'  => "O campo Escala é obrigatório!",
            'nrturno'        => "O campo Turno é obrigatório!",
        ];
        
        $erros = [];
        
        foreach ($camposObrigatorios as $campo => $mensagemErro) {
            if (empty($$campo)) {
                $erros[] = $mensagemErro;
            }
        }
        
        if (!empty($erros)) {
            throw new \Exception(implode(" ", $erros));
        }
        

        // SALÁRIO
        $alteSalario = ProcessamentoSolicitacaoUtil::novoAlteSalario($nrorg, $nrvinculom, $dtinimoviment, $nrocorrencia, $nrtipoaltera, $nrtpmodalidsal, $idtpsalario, $novoVlrSalario, $observacao);


        // ESCALA
        ProcessamentoSolicitacaoUtil::atualizaAlteEscala(
            $nrorg,
            $nrvinculom,
            $nrescalatrabm,
            DateUtil::subtraiIntervalo($dtiniescala,1,DateUtil::DIA)->format(DateUtil::FORMATO_BRASILEIRO),
            DateUtil::getDataAtual(true)->format(DateUtil::FORMATO_BRASILEIRO),
            Utils::getParamters()->getCdoperador()
        );
        
        $alteEscala = ProcessamentoSolicitacaoUtil::novoAlteEscala($nrorg, $nrvinculom, $nrturno, $nrmotivoescala, $nrescalatrabm, $dtiniescala, $observacao);

        // OCUPACAO
        if($nrocupacaom && $nrmotivoocupa && $nrtipoocupacao && $nrtpmodalidsal) {

            $this->atualizaAlteOcupa($nrvinculom, DateUtil::subtraiIntervalo($dtinimoviment, 1, DateUtil::DIA)->format(DateUtil::FORMATO_BRASILEIRO),DateUtil::getDataAtual(true)->format(DateUtil::FORMATO_BRASILEIRO),Utils::getParamters()->getCdoperador());

            $alteOcupacao = ProcessamentoSolicitacaoUtil::novoAlteOcupacao($nrorg, $nrvinculom, $nrocupacaom, $nrmotivoocupa, $nrtipoocupacao, $nrtpmodalidsal, $dtinimoviment, $observacao);
        }


        // Histórico do Vínculo
        $vinc = ProcessamentoSolicitacaoUtil::retornaUltimoHistoricoVinculoCompetencia($nrorg, $nrvinculom, $dtcompetencia);

        // MOVIMENTACAO
        if($nrestrutlegal) {
            $estrutura_nova_legal = Utils::getEntityManager()->getRepository(Repositories::ESTRUTURAM)->findOneBy(array("nrorg"=> $nrorg,"nrestruturam" => $nrestrutlegal));

            // Cria a Movimentação Legal
            ProcessamentoSolicitacaoUtil::novaMovimentacao($nrorg, $nrvinculom, $estrutura_nova_legal->getNrtipoestrutura(), $nrestrutlegal, $nrmotivotransf, $nrtipotransfer, $nrtpmovtransfm, $dtfimmoviment, $dtinimoviment, $observacao);
            $dtfimmovimentsaida = DateUtil::subtraiIntervalo($dtinimoviment, 1)->format('d/m/Y');
            $dtfimmovimentsaida = DateUtil::getDataDeString(substr($dtfimmovimentsaida,0,10), DateUtil::FORMATO_BRASILEIRO, true);
    
            $estrutura_anterior = Utils::getEntityManager()->getRepository(Repositories::ESTRUTURAM)->findOneBy(array("nrorg"=> $nrorg,"nrestruturam" => $vinc->getNrestrutlegal()));
    
            // Cria a Movimentação de saida
            ProcessamentoSolicitacaoUtil::novaMovimentacao($nrorg, $nrvinculom, $estrutura_anterior->getNrtipoestrutura(), $vinc->getNrestrutlegal(), $nrmotivotransf, $nrtipotransfer, "9", $dtfimmovimentsaida, $dtinimoviment, $observacao);
        }

        // Cria a Movimentação Gerencial
        if($nrestrutgeren) {
            $estrutura_nova_geren = Utils::getEntityManager()->getRepository(Repositories::ESTRUTURAM)->findOneBy(array("nrorg"=> $nrorg,"nrestruturam" => $nrestrutgeren));
            $nrtipotransfergeren = '2';

            ProcessamentoSolicitacaoUtil::novaMovimentacao($nrorg, $nrvinculom, $estrutura_nova_geren->getNrtipoestrutura(), $nrestrutgeren, $nrmotivotransf, $nrtipotransfergeren, $nrtpmovtransfm, $dtfimmoviment, $dtinimoviment, $observacao);
        }

        // Histórico do Vínculo
        if($vinc->getDtmescompetenc() == $dtcompetencia){
            $vinc->setDtultatu(DateUtil::getDataAtual());
            $vinc->setNrorgultatu(Utils::getParamters()->getNrorg());
            $vinc->setCdoperultatu(Utils::getParamters()->getCdoperador());
            $vinc->setNrescalatrabm($nrescalatrabm);
            $vinc->setNrturno($nrturno);

            if($nrestrutlegal) $vinc->setNrestrutlegal($nrestrutlegal);
            if($nrestrutgeren) $vinc->setNrestrutgeren($nrestrutgeren);
            if($nrocupacaom) $vinc->setNrcargo($nrocupacaom);


            Utils::getEntityManager()->persist($vinc);
            Utils::getEntityManager()->flush();
        }
        else {
            $estrutgeren = $nrestrutgeren ?? $vinc->getNrestrutgeren();
            $estrutlegal = $nrestrutlegal ?? $vinc->getNrestrutlegal();
            $nrcargo     = $nrocupacaom   ?? $vinc->getNrcargo();

            ProcessamentoSolicitacaoUtil::novoHistoricoVinculo($nrorg, $nrvinculom, $dtcompetencia, $vinc->getNrsitufuncm(), $nrcargo, $nrescalatrabm,
            $vinc->getIdcontribuisind(), $vinc->getNrdependir(), $vinc->getNrdependsfam(), $vinc->getDtbaseferias(), $vinc->getIdremuneracao(),
            $vinc->getIdtppagamento(), $estrutlegal, $estrutgeren, $vinc->getNrtpmodalidsal(), $vinc->getIdmultvinc(),
            $vinc->getNrfuncao(), $vinc->getCdcontribindividual(), $vinc->getNrtpmovtransfm(), $vinc->getDtfimcontrdetermin(),
            $vinc->getNrestrutsind(), $vinc->getConfidencial(), $nrturno
            );
        }

    }
    
    public function rejeitar() {}
    
    // Atualiza a data fim da ocupação anterior
    public function atualizaAlteOcupa($nrvinculom, $dtfimocupacao, $dtatual, $cdoperador) {
        $sql = <<<SQL
            UPDATE GPE_ALTEOCUPACAO
            SET DTFIMOCUPACAO = :P_DTFIMOCUPACAO,
            DTULTATU = :P_DTATUAL,
            NRORGULTATU = :P_NRORG,
            CDOPERULTATU = :P_CDOPERADOR
            WHERE DTINIOCUPACAO = (
                SELECT MAX(DTINIOCUPACAO) 
                FROM GPE_ALTEOCUPACAO 
                WHERE NRORG = :P_NRORG 
                AND NRVINCULOM = :P_NRVINCULOM 
                AND DTFIMOCUPACAO IS NULL
                )
            AND NRORG = :P_NRORG 
            AND NRVINCULOM = :P_NRVINCULOM 
            AND DTINIOCUPACAO >= :P_DTFIMOCUPACAO
            AND DTFIMOCUPACAO IS NULL
SQL;

        return Utils::getEntityManager()->getConnection()->prepare($sql)
            ->execute(array(
                "P_NRORG"             => Utils::getParamters()->getNrorg(),
                "P_DTFIMOCUPACAO"     => $dtfimocupacao,
                "P_NRVINCULOM"        => $nrvinculom,
                "P_DTATUAL"           => $dtatual,
                "P_CDOPERADOR"        => $cdoperador
            )
        );
    }
}