<?php

namespace Ems\Services;

use Ems\Repository\{ EquipeRepository, PeriodoRepository, GeralRepository };
use GuzzleHttp\Client;
use Ems\Util\Utils;

class EnderecoService {
    /** @var Client */
    protected $client;

    /** @var GeralRepository */
    protected $geralRepository;

    protected const CEP_API_URL = 'http://viacep.com.br/';
    protected const CEP_API_URI = 'ws/[CEP]/json';


    public function __construct(
        GeralRepository $geralRepository
    ) {
        $this->client = new Client([
            'base_uri' => self::CEP_API_URL
        ]);

        $this->geralRepository = $geralRepository;
    }

    public function pesquisarCep(
        string $cep
    ): array {
        $status = [
            'success' => true,
            'message' => null,
            'cepInfo'    => null
        ];

        $cepIsValid = self::validaCep($cep);

        if (!$cepIsValid) {
            $status['success'] = false;
            $status['message'] = 'CEP inválido!';

            return $status;
        }

        $cep = self::onlyNumbers($cep);

        $url = strtr(self::CEP_API_URI, [
            '[CEP]' => $cep,
        ]);

        $response = $this->client->get($url);

        $content = $response->getBody()->getContents();
        $body = json_decode($content, true);

        $status['cepInfo'] = $body;

        return $status;
    }

    public static function validaCep(
        string $cep
    ): bool {
        $isValid = preg_match('^\d{2}.?\d{3}-?\d{3}$^', $cep) === 1;
        return $isValid;
    }

    public static function onlyNumbers(
        string $text
    ): string {
        preg_match_all('!\d+!', $text, $matches);
        $numbers = implode('', $matches[0]);

        return $numbers;
    }

    public function getCdLogradouroByName(
        string $logradouro
    ) {
        $logradouro = Utils::removeCaracteresEspeciais($logradouro);
        $logradouro = str_replace('.', '', $logradouro);

        if (strcasecmp($logradouro, 'AV') || strcasecmp($logradouro, 'AVENIDA')) {
            return 'AV';
        }

        if (strcasecmp($logradouro, 'R') || strcasecmp($logradouro, 'RUA')) {
            return 'RUA';
        }

        $logradouroArray = $this->geralRepository->getLogradouroByName($logradouro);

        if ($logradouroArray) {
            return $logradouroArray['CDLOGRADOURO'];
        }

        return null;
    }
}