<?php

namespace Ems\Util;

/**
 *  Abstract Service
 * 
 *  Define rotinas comuns as entidades 'Service'. Entre elas:
 *      - Inclusão de dados: Service::set;
 *      - Alteração de dados: Service::update;
 *      - Remoção de dados: Service::update.
 * 
 */

abstract class Service {
    protected const ENTITY_PERSIST_INSTANCE_NAME = 'doctrineEntityUtil';

    protected abstract function getEntityRepositoryName(): string;

    protected abstract function getEntityKeysName(): array;

    protected function getEntityPersistInstance(): EntityPersistInstance {
        return Utils::getInstance(self::ENTITY_PERSIST_INSTANCE_NAME);
    }

    protected function set(
        array $row,
        ?array $config = []
    ) {
        return $this->getEntityPersistInstance()->setNewEntity(
            $this->getEntityRepositoryName(),
            $row,
            $config
        );
    }

    protected function update(
        array $row,
        ?array $config = []
    ) {
        return $this->getEntityPersistInstance()->updateEntity(
            $this->getEntityRepositoryName(),
            $this->getEntityKeysName(),
            $row,
            $config
        );
    }

    protected function remove(
        array $row,
        ?array $config = []
    ) {
        return $this->getEntityPersistInstance()->removeEntity(
            $this->getEntityRepositoryName(),
            $this->getEntityKeysName(),
            $row,
            $config
        );
    }
}