<?php

namespace HCM\FPA\Entity;

use HCM\Util\DateUtil;
use HCM\Util\Repositories;
use HCM\Util\TkEntity;

class FpaManadmagnetEntity extends TkEntity
{
    private $objManadMagnet;
    private $nrmanadmagnet;
    private $nrorg;
    private $mescompetencia;
    private $cdinscestrut;
    private $dtadmissaovinc;
    private $idtiporegistro;
    private $dslinharegistro;
    private $nrvinculom;
    private $nrpessoarespons;
    private $idativo;
    private $dtinclusao;
    private $nmpessoa;
    private $nrcpfpessoa;
    private $nrestrutlegal;
    private $vrbaseirrf;
    private $vrbaseps;
    private $vrrubr;
    private $nrcalculofolha;

    public function __construct($entityManager, $hcmParameters, $nrOrg, $mescompetencia, $cdInscEstrut){
        parent::__construct(null, $entityManager);
        $this->entityManager  = $entityManager;
        $this->hcmParameters  = $hcmParameters;
        $this->manadMagnet    = Repositories::FPA_MANADMAGNET;
        $this->nrorg          = $nrOrg;
        $this->mescompetencia = $mescompetencia;
        $this->cdinscestrut   = $cdInscEstrut;
        $this->dtinclusao     = DateUtil::getDataAtual();
        $this->idativo        = 'S';
    }
    
    public function retornaRegistros($tipoRegistro = null){
        try{
            $params = ['nrorg'          => $this->nrorg,
                       'mescompetencia' => DateUtil::getDataDeString($this->mescompetencia, DateUtil::FORMATO_BRASILEIRO, true),
                       'cdinscestrut'   => $this->cdinscestrut];
            
            if(!is_null($tipoRegistro))
                $params['idtiporegistro'] = $tipoRegistro;
            
            return $this->entityManager->getRepository(Repositories::FPA_MANADMAGNET)->findBy($params, ['idtiporegistro' => 'ASC','nrvinculom' => 'ASC']);
        } catch(\Exception $e) {
            throw new \Exception('Não foi possivel localizar registros do MANAD no magnético. O erro foi: '.$e->getMessage());
        }
    }

    public function qdeRegistros(){
        $cont = 0;
        try{
            $registros = $this->retornaRegistros();
            $cont = count($registros);
        } catch(\Exception $e) {
            throw new \Exception('Não foi possivel localizar registros do MANAD no magnético. O erro foi: '.$e->getMessage());
        }
        return $cont;
    }

    public function addRegCabecal($cdInscEstrut, $nrPessoaRespons, $idAtivo){
        $this->cdinscestrut    = $cdInscEstrut;
        $this->nrpessoarespons = $nrPessoaRespons;
        $this->idativo         = $idAtivo;
    }
    
    public function addRegVinculo($nrCpf, $nmpessoa, $dtAdmissao, $idTipoRegistro, $registro, $nrestrutlegal = null, $nrvinculom,
                                  $vrbaseirrf = null, $vrbaseps = null, $vrrubr = null, $nrcalculofolha = null){
        $this->nrvinculom     = $nrvinculom;
        $this->nrestrutlegal  = $nrestrutlegal;
        $this->nrcpfpessoa    = $nrCpf;
        $this->nmpessoa       = $nmpessoa;
        $this->dtadmissaovinc = $dtAdmissao;
        $this->idtiporegistro = $idTipoRegistro;
        $this->vrbaseirrf     = $vrbaseirrf;
        $this->vrbaseps       = $vrbaseps;
        $this->vrrubr         = $vrrubr;
        $this->nrcalculofolha = $nrcalculofolha;
        $this->dslinharegistro .= is_null($this->dslinharegistro) ? $registro : "\n".$registro;
    }

    public function fimAddRegVinculo(){
        $this->setAllToObj();
        $this->persistir();     
        $this->zeraRegVinculo();    
    }

    public function excluirCompetencia(){
        $cont = 0;
        try{
            $cont = $this->qdeRegistros();
            
            $query = $this->entityManager->getConnection()->prepare("
                DELETE FROM FPA_MANADMAGNET
                 WHERE NRORG          = :P_NRORG
                   AND MESCOMPETENCIA = :P_MESCOMPETENCIA
                   AND CDINSCESTRUT   = :P_CDINSCESTRUT");
    
            $query->execute([
                "P_NRORG"          => $this->nrorg,
                "P_MESCOMPETENCIA" => $this->mescompetencia->format('d/m/Y'),
                "P_CDINSCESTRUT"   => $this->cdinscestrut]);
            
        } catch(\Exception $e) {
            throw new \Exception('Não foi possivel retroceder o MANAD no magnético. O erro foi: '.$e->getMessage());
        }
        return $cont;
    }

    private function setAllToObj(){ 
        $this->objManadMagnet = new $this->manadMagnet;
        $this->nrmanadmagnet = \HCM\Util\NovoCodigo::geraCodigo($this->nrorg,'FPA_MANADMAGNET'.$this->nrorg, 1, 22, true ); 
        $this->objManadMagnet->setNrmanadmagnet($this->nrmanadmagnet);
        $this->objManadMagnet->setNrorg($this->nrorg);
        $this->objManadMagnet->setMescompetencia(DateUtil::getDataDeString($this->mescompetencia, DateUtil::FORMATO_BRASILEIRO, true));
        $this->objManadMagnet->setCdinscestrut($this->cdinscestrut);
        $this->objManadMagnet->setDtadmissaovinc(DateUtil::getDataDeString($this->dtadmissaovinc, DateUtil::FORMATO_BRASILEIRO, true));
        $this->objManadMagnet->setIdtiporegistro($this->idtiporegistro);
        $this->objManadMagnet->setDslinharegistro($this->dslinharegistro);
        $this->objManadMagnet->setNrvinculom($this->nrvinculom);
        $this->objManadMagnet->setNrpessoarespons($this->nrpessoarespons);
        $this->objManadMagnet->setIdativo($this->idativo);
        $this->objManadMagnet->setDtinclusao($this->dtinclusao);
        $this->objManadMagnet->setDtultatu(DateUtil::getDataAtual());
        $this->objManadMagnet->setNrorginclusao($this->nrorg);
        $this->objManadMagnet->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $this->objManadMagnet->setNrorgultatu($this->nrorg);
        $this->objManadMagnet->setCdoperultatu($this->hcmParameters->getCdoperador());
        $this->objManadMagnet->setNmpessoa($this->nmpessoa);
        $this->objManadMagnet->setNrcpfpessoa($this->nrcpfpessoa);
        $this->objManadMagnet->setNrestrutlegal($this->nrestrutlegal);
        $this->objManadMagnet->setVrbaseirrf($this->vrbaseirrf);
        $this->objManadMagnet->setVrbaseps($this->vrbaseps);
        $this->objManadMagnet->setVrrubr($this->vrrubr);
        $this->objManadMagnet->setNrcalculofolha($this->nrcalculofolha);
    }
    
    private function zeraRegVinculo(){
        $this->objManadMagnet = $this->dtadmissaovinc = $this->idtiporegistro = $this->dslinharegistro = $this->nrvinculom = $this->nmpessoa        =
        $this->nrcpfpessoa    = $this->nrestrutlegal  = $this->vrbaseirrf     = $this->vrbaseps        = $this->vrrubr     = $this->nrcalculofolha  = null;
    }
    
    private function persistir(){
        try{
            $this->entityManager->persist($this->objManadMagnet);
            $this->entityManager->flush();
        } catch(\Exception $e) {
            throw new \Exception('Não foi possivel salvar o MANAD no magnético. O erro foi: '.$e->getMessage());
        }
    }
}