<?php

namespace HCM\FPA\Services;

use Doctrine\ORM\EntityManager;
use Teknisa\Model\Gestaopessoas\Entities\FpaIncidevento;
use HCM\Util\DateUtil;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\Repositories;
use Geral\Util\NovoCodigo;

class IncidenciaEventoService {

    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters) {
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
    }
 
    public function verificaDuplicidade($tipoincide, $eventom){
        $incidencia = $this->entityManager->getRepository(Repositories::FPA_INCIDEVENTO)->findBy(array(
            'nrorg' => $this->hcmParameters->getNrorgTrabalho(),
            'nrtipoincide' => $tipoincide,
            'nreventom' => $eventom));
            return $incidencia;
    }
    
    
    public function cadAutoIncEvt($tipoincide, $eventom, $incidevento) {
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        
        $this->entityManager->beginTransaction();
        
        try {
            $existe = $this->verificaExistenciaRegistro($nrorg, $tipoincide, $eventom, $incidevento);
            if(!$existe){
                $codigo = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'FPA_INCIDEVENTO', 1, 12, true);
                $obj = new FpaIncidevento;
                $obj->setNrorg($this->hcmParameters->getNrorgTrabalho());
                $obj->setDtinclusao(DateUtil::getDataAtual());
                $obj->setCdoperinclusao($this->hcmParameters->getCdoperador());
                $obj->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
                $obj->setDtultatu(DateUtil::getDataAtual());
                $obj->setCdoperultatu($this->hcmParameters->getCdoperador());
                $obj->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
                $obj->setDtmescompetenc(DateUtil::getDataDeString($this->hcmParameters->getDtCompetencia(), DateUtil::FORMATO_BRASILEIRO, true));
                $obj->setNreventom($eventom);
                $obj->setIdincidevento($incidevento);
                $obj->setNrtipoincide($tipoincide);
                $obj->setNrincidevento($codigo);
                $this->entityManager->persist($obj);
                $this->entityManager->flush($obj);
            }
            $this->entityManager->commit();
        } catch(\Exception $e) {
            $this->entityManager->rollback();
            throw new \Exception($e->getMessage());
        }
    }
    
    public function verificaExistenciaRegistro($nrorg, $tipoincide, $eventom, $incidevento){ // evita registros duplicados
    
        $incidenciaEvento = $this->entityManager->getRepository(Repositories::FPA_INCIDEVENTO)->findBy(array('nrorg' => $nrorg,
            'nrtipoincide' => $tipoincide, 'nreventom' => $eventom,'idincidevento' => $incidevento,
            'dtmescompetenc' => DateUtil::getDataDeString($this->hcmParameters->getDtCompetencia(), DateUtil::FORMATO_BRASILEIRO, true)));
            
        $mesmaIncidencia = $this->entityManager->getRepository(Repositories::FPA_INCIDEVENTO)->findBy(array('nrorg' => array($this->hcmParameters->getNrorgPadrao(), $this->hcmParameters->getNrorgTrabalho()),
            'nrtipoincide' => $tipoincide, 'nreventom' => $eventom));
        
        if(count($incidenciaEvento)>0 or count($mesmaIncidencia)>0){
            return true;
        } else {
            return false;
        }
    }
    
    /**
     * Rotina que visa a preparação para verificação e a criação se necessário de incidências, realizada na rotina 'atualizaIncidencias'
     * no arquivo IncidenciaEventoService.php
     * 
     * @param int $newCode Variável que representa o número do novo evento gerado
     */
    public function atualizaIncidencias($incideinss, $incidefgts, $incideirrf, $incide13, $incideferias, $incideCompFerias, $incideCompProv, $incidePeric, $incideInsalub, $incidePensAlim, $incidePensAlimFe, $incideDSRHoraExt, $incideDSRAdicNot, $incideCSind, $incideSalarioFamilia, $incideDeducaoINSS, $nreventom, $nreventoh, $idtipoevento, $newCode, $isNew, $eventoh){
        try {
            $nrincidencias = [];
            $dtemescompetencia = $this->hcmParameters->getDtCompetencia();
            $nrorg = $this->hcmParameters->getNrorgTrabalho();
            
            $opcaoIncidencia = 'N';
            if($idtipoevento == 'PROVENTO')
                $opcaoIncidencia = '+';
            else if($idtipoevento == 'DESCONTO')
                $opcaoIncidencia = '-';
                
            if(empty($row['NREVENTOM']))
                $row['NREVENTOM'] = $newCode; //se o evento é novo, a rotina existente no CrudController já gerou o código
            
            $parametros = array("NREVENTOM" => $nreventom, "NREVENTOH" => $nreventoh, "DTMESCOMPETENC" => $dtemescompetencia, "NRORG" => $nrorg, "IDINCIDEVENTO" => $opcaoIncidencia);
            
            if(!empty($eventoh)){
                $estadoTiposIncidenciaEventoAnterior = array('NRINCIDCP' => $eventoh->getNrincidcp(), 'NRINCIDIRRF' => $eventoh->getNrincidirrf(), 'NRINCIDFGTS' => $eventoh->getNrincidfgts(),
                                                             'IDREPDECTERC' => $eventoh->getIdrepdecterc(), 'IDREPFERIAS' => $eventoh->getIdrepferias(), 'IDCOMPSALFER' => $eventoh->getIdcompsalfer(), 
                                                             'IDCOMPSALPROV' => $eventoh->getIdcompsalprov(), 'IDPROPORCEVENTO' => $eventoh->getIdproporcevento(), 'IDPAGTOOBRIG' => $eventoh->getIdpagtoobrig(),
                                                             'IDREFADECTERC' => $eventoh->getIdrefadecterc(), 'IDREFAFERIAS' => $eventoh->getIdrefaferias(), 'IDREFADIANT' => $eventoh->getIdrefadiant(),
                                                             'IDINCIDEPERIC' => $eventoh->getIdincideperic(), 'IDINCIDEINSALUB' => $eventoh->getIdincideinsalub(), 'IDINCIDEPENSALIM' => $eventoh->getIdincidepensalim(),
                                                             'IDINCIDEPENSALIMFE' => $eventoh->getIdincidepensalimfe(), 'IDINCIDEDSRHORAEXT' => $eventoh->getIdincidedsrhoraext(), 'IDREFHORAEXTRA' => $eventoh->getIdrefhoraextra(),
                                                             'IDINCIDEDSRADICNOT' => $eventoh->getIdincidedsradicnot(), 'IDREFADICIONOTURN' => $eventoh->getIdrefadicionoturn(), 'IDREFADICTSERV' => $eventoh->getIdrefadictserv(),
                                                             'IDINCIDECSIND' => $eventoh->getIdincidecsind(), 'IDSALARIOFAMILIA' => $eventoh->getIdsalariofamilia(), 'IDREFSEGUROVIDA' => $eventoh->getIdrefsegurovida(),
                                                             'IDDEDUCAOINSS' => $eventoh->getIddeducaoinss(),
                                                            );
            } else {
                $estadoTiposIncidenciaEventoAnterior = null;
            }
            
            $this->gerenciaIncidencias($parametros, $isNew, $incideinss, $incidefgts, $incideirrf, $incide13, $incideferias, $incideCompFerias, $incideCompProv, $incidePeric, $incideInsalub, $incidePensAlim, $incidePensAlimFe, $incideDSRHoraExt, $incideDSRAdicNot, $incideCSind, $incideSalarioFamilia, $incideDeducaoINSS, $estadoTiposIncidenciaEventoAnterior);
        } catch(\Exception $e){
            throw $e;
        }
    }
    
    /**
     * Rotina que visa a verificação e a criação se necessário de incidências em decorrência da criação ou edição de eventos
     * @param array('NREVENTOM', 'DTMESCOMPETENC', 'NRORG', 'IDINCIDEVENTO')                    $parametros                         : Array que carrega informações para a criação dos registros das incidências
     * @param bool                                                                              $isNew                              : Flag que identifica se o registro é novo ou se é um processo de edição de um registro existente
     * @param array('NRINCIDCP', 'NRINCIDIRRF', 'NRINCIDFGTS', 'IDREPDECTERC', 'IDREPFERIAS')   $estadoTiposIncidenciaEventoAnterior: Array que possui os valores dos campos de incidência no último momento antes de uma edição (se for o caso)
     * @return void
     * 
     */
    public function gerenciaIncidencias($parametros, $isNew, $incideinss, $incidefgts, $incideirrf, $incide13, $incideferias, $incideCompFerias, $incideCompProv, $incidePeric, $incideInsalub, $incidePensAlim, $incidePensAlimFe, $incideDSRHoraExt, $incideDSRAdicNot, $incideCSind, $incideSalarioFamilia, $incideDeducaoINSS, $estadoTiposIncidenciaEventoAnterior){
        $novoEvento = empty($estadoTiposIncidenciaEventoAnterior);
        if($novoEvento){
            $nrincidencias = $this->retornaIncidenciasInclusao($incideinss, $incidefgts, $incideirrf, $incide13, $incideferias, $incideCompFerias, $incideCompProv, $incidePeric, $incideInsalub, $incidePensAlim, $incidePensAlimFe, $incideDSRHoraExt, $incideDSRAdicNot, $incideCSind, $incideSalarioFamilia, $incideDeducaoINSS);
            $this->salvaIncidencias($nrincidencias, $parametros);
        } else {
            $incidenciasASeremIncluidas = $this->retornaIncidenciasASeremIncluidas($incideinss, $incidefgts, $incideirrf, $incide13, $incideferias, $incideCompFerias, $incideCompProv, $incidePeric, $incideInsalub, $incidePensAlim, $incidePensAlimFe, $incideDSRHoraExt, $incideDSRAdicNot, $incideCSind, $incideSalarioFamilia, $incideDeducaoINSS, $parametros, $estadoTiposIncidenciaEventoAnterior);
            $incidenciasASeremExcluidas = $this->retornaIncidenciasASeremExcluidas($incideinss, $incidefgts, $incideirrf, $incide13, $incideferias, $incideCompFerias, $incideCompProv, $incidePeric, $incideInsalub, $incidePensAlim, $incidePensAlimFe, $incideDSRHoraExt, $incideDSRAdicNot, $incideCSind, $incideSalarioFamilia, $incideDeducaoINSS, $parametros, $estadoTiposIncidenciaEventoAnterior);
            
            if(!empty($incidenciasASeremExcluidas)){
                $this->deleteIncidencias($incidenciasASeremExcluidas, $parametros);
            }
            
            if(!empty($incidenciasASeremIncluidas)){
                $this->salvaIncidencias($incidenciasASeremIncluidas, $parametros);
            }
            $this->atualizaHistoricosIncidencias($incideinss, $incidefgts, $incideirrf, $incide13, $incideferias, $incideCompFerias, $incideCompProv, $incidePeric, $incideInsalub, $incidePensAlim, $incidePensAlimFe, $incideDSRHoraExt, $incideDSRAdicNot, $incideCSind, $incideSalarioFamilia, $incideDeducaoINSS, $parametros, $estadoTiposIncidenciaEventoAnterior);
        }
    }
    
    /**
     * Rotina que verifica e retorna quais campos de incidência foram alterados na última edição para serem salvos e retorna um array com os números dos tipos de incidência a serem inseridas
     * @param array("NREVENTOM", "DTMESCOMPETENC", "NRORG", "IDINCIDEVENTO")                      $parametros                         : Array que carrega informações para a criação dos registros das incidências
     * @param array('NRINCIDCP', 'NRINCIDIRRF', 'NRINCIDFGTS', 'IDREPDECTERC', 'IDREPFERIAS')   $estadoTiposIncidenciaEventoAnterior: Array que possui os valores dos campos de incidência no último momento antes de uma edição (se for o caso)
     * @return array
     * 
     */
    public function retornaIncidenciasASeremIncluidas($incideinss, $incidefgts, $incideirrf, $incide13, $incideferias, $incideCompFerias, $incideCompProv, $incidePeric, $incideInsalub, $incidePensAlim, $incidePensAlimFe, $incideDSRHoraExt, $incideDSRAdicNot, $incideCSind, $incideSalarioFamilia, $incideDeducaoINSS, $parametros, $estadoTiposIncidenciaEventoAnterior){
        $incidenciasASeremIncluidas = array();
        if($incideinss == 2 && $this->seTipoIncidenciaEstaMarcado($estadoTiposIncidenciaEventoAnterior, 'NRINCIDCP', 1)) {
            $incidenciasAindaNaoCadastradas = $this->buscaIncidenciasAindaNaoCadastradas(array(2, 5), $parametros);
            if($incidenciasAindaNaoCadastradas){
                $incidenciasASeremIncluidas = array_merge($incidenciasASeremIncluidas, $incidenciasAindaNaoCadastradas);
            }
        }
        
        if($incidefgts == 2 && $this->seTipoIncidenciaEstaMarcado($estadoTiposIncidenciaEventoAnterior, 'NRINCIDFGTS', 1)){
            $incidenciasAindaNaoCadastradas = $this->buscaIncidenciasAindaNaoCadastradas(array(3, 6), $parametros);
            if($incidenciasAindaNaoCadastradas){
                $incidenciasASeremIncluidas = array_merge($incidenciasASeremIncluidas, $incidenciasAindaNaoCadastradas);
            }
        }
        
        if($incideirrf == 2 && $this->seTipoIncidenciaEstaMarcado($estadoTiposIncidenciaEventoAnterior, 'NRINCIDIRRF', 1)){
            $incidenciasAindaNaoCadastradas = $this->buscaIncidenciasAindaNaoCadastradas(array(1, 4), $parametros);
            if($incidenciasAindaNaoCadastradas){
                $incidenciasASeremIncluidas = array_merge($incidenciasASeremIncluidas, $incidenciasAindaNaoCadastradas);
            }
        } 
        
        if($incide13 == 'S' && $this->seTipoIncidenciaEstaMarcado($estadoTiposIncidenciaEventoAnterior, 'IDREPDECTERC', 'N')){
            $incidenciasAindaNaoCadastradas = $this->buscaIncidenciasAindaNaoCadastradas(array(25), $parametros);
            if($incidenciasAindaNaoCadastradas){
                $incidenciasASeremIncluidas = array_merge($incidenciasASeremIncluidas, $incidenciasAindaNaoCadastradas);
            }
        }
        
        if($incideferias == 'S' && $this->seTipoIncidenciaEstaMarcado($estadoTiposIncidenciaEventoAnterior, 'IDREPFERIAS', 'N')){
            $incidenciasAindaNaoCadastradas = $this->buscaIncidenciasAindaNaoCadastradas(array(24), $parametros);
            if($incidenciasAindaNaoCadastradas){
                $incidenciasASeremIncluidas = array_merge($incidenciasASeremIncluidas, $incidenciasAindaNaoCadastradas);
            }
        }
        
        if($incideCompFerias == 'S' && $this->seTipoIncidenciaEstaMarcado($estadoTiposIncidenciaEventoAnterior, 'IDCOMPSALFER', 'N')){
            $incidenciasAindaNaoCadastradas = $this->buscaIncidenciasAindaNaoCadastradas(array(210), $parametros);
            if($incidenciasAindaNaoCadastradas){
                $incidenciasASeremIncluidas = array_merge($incidenciasASeremIncluidas, $incidenciasAindaNaoCadastradas);
            }
        }
        
        if($incideCompProv == 'S' && $this->seTipoIncidenciaEstaMarcado($estadoTiposIncidenciaEventoAnterior, 'IDCOMPSALPROV', 'N')){
            $incidenciasAindaNaoCadastradas = $this->buscaIncidenciasAindaNaoCadastradas(array(212), $parametros);
            if($incidenciasAindaNaoCadastradas){
                $incidenciasASeremIncluidas = array_merge($incidenciasASeremIncluidas, $incidenciasAindaNaoCadastradas);
            }
        }
        
        if($incidePeric == 'S' && $this->seTipoIncidenciaEstaMarcado($estadoTiposIncidenciaEventoAnterior, 'IDINCIDEPERIC', 'N')){
            $incidenciasAindaNaoCadastradas = $this->buscaIncidenciasAindaNaoCadastradas(array(29), $parametros);
            if($incidenciasAindaNaoCadastradas){
                $incidenciasASeremIncluidas = array_merge($incidenciasASeremIncluidas, $incidenciasAindaNaoCadastradas);
            }
        }
        
        if($incideInsalub == 'S' && $this->seTipoIncidenciaEstaMarcado($estadoTiposIncidenciaEventoAnterior, 'IDINCIDEINSALUB', 'N')){
            $incidenciasAindaNaoCadastradas = $this->buscaIncidenciasAindaNaoCadastradas(array(28), $parametros);
            if($incidenciasAindaNaoCadastradas){
                $incidenciasASeremIncluidas = array_merge($incidenciasASeremIncluidas, $incidenciasAindaNaoCadastradas);
            }
        }
        
        if($incidePensAlim == 'S' && $this->seTipoIncidenciaEstaMarcado($estadoTiposIncidenciaEventoAnterior, 'IDINCIDEPENSALIM', 'N')){
            $incidenciasAindaNaoCadastradas = $this->buscaIncidenciasAindaNaoCadastradas(array(36), $parametros);
            if($incidenciasAindaNaoCadastradas){
                $incidenciasASeremIncluidas = array_merge($incidenciasASeremIncluidas, $incidenciasAindaNaoCadastradas);
            }
        }
        
        if($incidePensAlimFe == 'S' && $this->seTipoIncidenciaEstaMarcado($estadoTiposIncidenciaEventoAnterior, 'IDINCIDEPENSALIMFE', 'N')){
            $incidenciasAindaNaoCadastradas = $this->buscaIncidenciasAindaNaoCadastradas(array(36, 132), $parametros);
            if($incidenciasAindaNaoCadastradas){
                $incidenciasASeremIncluidas = array_merge($incidenciasASeremIncluidas, $incidenciasAindaNaoCadastradas);
            }
        }
        
        if($incideDSRAdicNot == 'S' && $this->seTipoIncidenciaEstaMarcado($estadoTiposIncidenciaEventoAnterior, 'IDINCIDEDSRADICNOT', 'N')){
            $incidenciasAindaNaoCadastradas = $this->buscaIncidenciasAindaNaoCadastradas(array(58), $parametros);
            if($incidenciasAindaNaoCadastradas){
                $incidenciasASeremIncluidas = array_merge($incidenciasASeremIncluidas, $incidenciasAindaNaoCadastradas);
            }
        }
        
        if($incideCSind == 'S' && $this->seTipoIncidenciaEstaMarcado($estadoTiposIncidenciaEventoAnterior, 'IDINCIDECSIND', 'N')){
            $incidenciasAindaNaoCadastradas = $this->buscaIncidenciasAindaNaoCadastradas(array(32), $parametros);
            if($incidenciasAindaNaoCadastradas){
                $incidenciasASeremIncluidas = array_merge($incidenciasASeremIncluidas, $incidenciasAindaNaoCadastradas);
            }
        }
        
        if($incideSalarioFamilia == 'S' && $this->seTipoIncidenciaEstaMarcado($estadoTiposIncidenciaEventoAnterior, 'IDSALARIOFAMILIA', 'N')){
            $incidenciasAindaNaoCadastradas = $this->buscaIncidenciasAindaNaoCadastradas(array(33), $parametros);
            if($incidenciasAindaNaoCadastradas){
                $incidenciasASeremIncluidas = array_merge($incidenciasASeremIncluidas, $incidenciasAindaNaoCadastradas);
            }
        }
        
        if($incideDeducaoINSS == 'S' && $this->seTipoIncidenciaEstaMarcado($estadoTiposIncidenciaEventoAnterior, 'IDDEDUCAOINSS', 'N')){
            $incidenciasAindaNaoCadastradas = $this->buscaIncidenciasAindaNaoCadastradas(array(224), $parametros);
            if($incidenciasAindaNaoCadastradas){
                $incidenciasASeremIncluidas = array_merge($incidenciasASeremIncluidas, $incidenciasAindaNaoCadastradas);
            }
        }
        
        return $incidenciasASeremIncluidas;
    }
    
    /**
     * Rotina que verifica e retorna quais campos de incidência foram alterados na última edição para serem excluídos e retorna um array com os números dos tipos de incidência a serem apagados
     * @param array("NREVENTOM", "DTMESCOMPETENC", "NRORG", "IDINCIDEVENTO")                      $parametros                         : Array que carrega informações para a criação dos registros das incidências
     * * @param array('NRINCIDCP', 'NRINCIDIRRF', 'NRINCIDFGTS', 'IDREPDECTERC', 'IDREPFERIAS')   $estadoTiposIncidenciaEventoAnterior: Array que possui os valores dos campos de incidência no último momento antes de uma edição (se for o caso)
     * @return array
     * 
     */
    public function retornaIncidenciasASeremExcluidas($incideinss, $incidefgts, $incideirrf, $incide13, $incideferias, $incideCompFerias, $incideCompProv, $incidePeric, $incideInsalub, $incidePensAlim, $incidePensAlimFe, $incideDSRHoraExt, $incideDSRAdicNot, $incideCSind, $incideSalarioFamilia, $incideDeducaoINSS, $parametros, $estadoTiposIncidenciaEventoAnterior){
        $incidenciasASeremExcluidas = array();
        if($incideinss == 1 && $this->seTipoIncidenciaEstaMarcado($estadoTiposIncidenciaEventoAnterior, 'NRINCIDCP', 2)) {
            $incidenciasAindaNaoExcluidas = $this->buscaIncidenciasASeremExcluidas(array(2, 5, 8, 11, 17, 20), $parametros);
            if($incidenciasAindaNaoExcluidas){
                $incidenciasASeremExcluidas = array_merge($incidenciasASeremExcluidas, $incidenciasAindaNaoExcluidas);
            }
        }
        
        if($incidefgts == 1 && $this->seTipoIncidenciaEstaMarcado($estadoTiposIncidenciaEventoAnterior, 'NRINCIDFGTS', 2)){
            $incidenciasAindaNaoExcluidas = $this->buscaIncidenciasASeremExcluidas(array(3, 6, 9, 12, 18, 21), $parametros);
            if($incidenciasAindaNaoExcluidas){
                $incidenciasASeremExcluidas = array_merge($incidenciasASeremExcluidas, $incidenciasAindaNaoExcluidas);
            }
        }
        
        if($incideirrf == 1 && $this->seTipoIncidenciaEstaMarcado($estadoTiposIncidenciaEventoAnterior, 'NRINCIDIRRF', 2)){
            $incidenciasAindaNaoExcluidas = $this->buscaIncidenciasASeremExcluidas(array(1, 4, 7, 10, 16, 19), $parametros);
            if($incidenciasAindaNaoExcluidas){
                $incidenciasASeremExcluidas = array_merge($incidenciasASeremExcluidas, $incidenciasAindaNaoExcluidas);
            }
        }
        
        if($incide13 == 'N' && $this->seTipoIncidenciaEstaMarcado($estadoTiposIncidenciaEventoAnterior, 'IDREPDECTERC', 'S')){
            $incidenciasAindaNaoExcluidas = $this->buscaIncidenciasASeremExcluidas(array(25), $parametros);
            if($incidenciasAindaNaoExcluidas){
                $incidenciasASeremExcluidas = array_merge($incidenciasASeremExcluidas, $incidenciasAindaNaoExcluidas);
            }
        }
        
        if($incideferias == 'N' && $this->seTipoIncidenciaEstaMarcado($estadoTiposIncidenciaEventoAnterior, 'IDREPFERIAS', 'S')){
            $incidenciasAindaNaoExcluidas = $this->buscaIncidenciasASeremExcluidas(array(24), $parametros);
            if($incidenciasAindaNaoExcluidas){
                $incidenciasASeremExcluidas = array_merge($incidenciasASeremExcluidas, $incidenciasAindaNaoExcluidas);
            }
        }
        if($incideCompFerias == 'N' && $this->seTipoIncidenciaEstaMarcado($estadoTiposIncidenciaEventoAnterior, 'IDCOMPSALFER', 'S')){
            $incidenciasAindaNaoExcluidas = $this->buscaIncidenciasASeremExcluidas(array(210), $parametros);
            if($incidenciasAindaNaoExcluidas){
                $incidenciasASeremExcluidas = array_merge($incidenciasASeremExcluidas, $incidenciasAindaNaoExcluidas);
            }
        }
        
        if($incideCompProv == 'N' && $this->seTipoIncidenciaEstaMarcado($estadoTiposIncidenciaEventoAnterior, 'IDCOMPSALPROV', 'S')){
            $incidenciasAindaNaoExcluidas = $this->buscaIncidenciasASeremExcluidas(array(212), $parametros);
            if($incidenciasAindaNaoExcluidas){
                $incidenciasASeremExcluidas = array_merge($incidenciasASeremExcluidas, $incidenciasAindaNaoExcluidas);
            }
        }
        
        if($incidePeric == 'N' && $this->seTipoIncidenciaEstaMarcado($estadoTiposIncidenciaEventoAnterior, 'IDINCIDEPERIC', 'S')){
            $incidenciasAindaNaoExcluidas = $this->buscaIncidenciasASeremExcluidas(array(29), $parametros);
            if($incidenciasAindaNaoExcluidas){
                $incidenciasASeremExcluidas = array_merge($incidenciasASeremExcluidas, $incidenciasAindaNaoExcluidas);
            }
        }
        
        if($incideInsalub == 'N' && $this->seTipoIncidenciaEstaMarcado($estadoTiposIncidenciaEventoAnterior, 'IDINCIDEINSALUB', 'S')){
            $incidenciasAindaNaoExcluidas = $this->buscaIncidenciasASeremExcluidas(array(28), $parametros);
            if($incidenciasAindaNaoExcluidas){
                $incidenciasASeremExcluidas = array_merge($incidenciasASeremExcluidas, $incidenciasAindaNaoExcluidas);
            }
        }
        
        if($incidePensAlim == 'N' && $this->seTipoIncidenciaEstaMarcado($estadoTiposIncidenciaEventoAnterior, 'IDINCIDEPENSALIM', 'S')){
            $incidenciasAindaNaoExcluidas = $this->buscaIncidenciasASeremExcluidas(array(36, 132), $parametros);
            if($incidenciasAindaNaoExcluidas){
                $incidenciasASeremExcluidas = array_merge($incidenciasASeremExcluidas, $incidenciasAindaNaoExcluidas);
            }
        }
        
        if($incidePensAlimFe == 'N' && $this->seTipoIncidenciaEstaMarcado($estadoTiposIncidenciaEventoAnterior, 'IDINCIDEPENSALIMFE', 'S')){
            $incidenciasAindaNaoExcluidas = $this->buscaIncidenciasASeremExcluidas(array(36, 132), $parametros);
            if($incidenciasAindaNaoExcluidas){
                $incidenciasASeremExcluidas = array_merge($incidenciasASeremExcluidas, $incidenciasAindaNaoExcluidas);
            }
        }
        
        if($incideDSRAdicNot == 'N' && $this->seTipoIncidenciaEstaMarcado($estadoTiposIncidenciaEventoAnterior, 'IDINCIDEDSRADICNOT', 'S')){
            $incidenciasAindaNaoExcluidas = $this->buscaIncidenciasASeremExcluidas(array(58), $parametros);
            if($incidenciasAindaNaoExcluidas){
                $incidenciasASeremExcluidas = array_merge($incidenciasASeremExcluidas, $incidenciasAindaNaoExcluidas);
            }
        }
        
        if($incideCSind == 'N' && $this->seTipoIncidenciaEstaMarcado($estadoTiposIncidenciaEventoAnterior, 'IDINCIDECSIND', 'S')){
            $incidenciasAindaNaoExcluidas = $this->buscaIncidenciasASeremExcluidas(array(32), $parametros);
            if($incidenciasAindaNaoExcluidas){
                $incidenciasASeremExcluidas = array_merge($incidenciasASeremExcluidas, $incidenciasAindaNaoExcluidas);
            }
        }
        
        if($incideSalarioFamilia == 'N' && $this->seTipoIncidenciaEstaMarcado($estadoTiposIncidenciaEventoAnterior, 'IDSALARIOFAMILIA', 'S')){
            $incidenciasAindaNaoExcluidas = $this->buscaIncidenciasASeremExcluidas(array(33), $parametros);
            if($incidenciasAindaNaoExcluidas){
                $incidenciasASeremExcluidas = array_merge($incidenciasASeremExcluidas, $incidenciasAindaNaoExcluidas);
            }
        }
        
        if($incideDeducaoINSS == 'N' && $this->seTipoIncidenciaEstaMarcado($estadoTiposIncidenciaEventoAnterior, 'IDDEDUCAOINSS', 'S')){
            $incidenciasAindaNaoExcluidas = $this->buscaIncidenciasASeremExcluidas(array(224), $parametros);
            if($incidenciasAindaNaoExcluidas){
                $incidenciasASeremExcluidas = array_merge($incidenciasASeremExcluidas, $incidenciasAindaNaoExcluidas);
            }
        }
        
        return $incidenciasASeremExcluidas;
    }
    
    public function seTipoIncidenciaEstaMarcado($estadoTiposIncidenciaEventoAnterior, $incidencia, $valor){
        if($estadoTiposIncidenciaEventoAnterior[$incidencia] == $valor){
            return true;
        } else {
            return false;
        }
         
    }
    
    /**
     * 
     * Função que verifica quais incidências deverão ser incluídas no processo de inclusão do evento
     * 
     */
    public function retornaIncidenciasInclusao($incideinss, $incidefgts, $incideirrf, $incide13, $incideferias, $incideCompFerias, $incideCompProv, $incidePeric, $incideInsalub, $incidePensAlim, $incidePensAlimFe, $incideDSRHoraExt, $incideDSRAdicNot, $incideCSind, $incideSalarioFamilia, $incideDeducaoINSS){
        $nrincidencias = [];
        if($incideinss == 2){
            array_push($nrincidencias, 2,5);
        } 
        
        if($incidefgts == 2){
            array_push($nrincidencias, 6,3);
        }
        
        if($incideirrf == 2){
            array_push($nrincidencias, 1,4);
        }
        
        if($incide13 == 'S'){
            array_push($nrincidencias, 25);
        }
        
        if($incideferias == 'S'){
            array_push($nrincidencias, 24);
        }
        
        if($incideCompFerias == 'S'){
            array_push($nrincidencias, 210);
        }
        
        if($incideCompProv == 'S'){
            array_push($nrincidencias, 212);
        }
        
        if($incidePeric == 'S'){
            array_push($nrincidencias, 29);
        }
        
        if($incideInsalub == 'S'){
            array_push($nrincidencias, 28);
        }
        
        if($incidePensAlim == 'S'){
            array_push($nrincidencias, 36);
        }
        
        if($incidePensAlimFe == 'S'){
            array_push($nrincidencias, 36, 132);
        }
        
        if($incideDSRHoraExt == 'S'){
            array_push($nrincidencias, 57);
        }
        
        if($incideDSRAdicNot == 'S'){
            array_push($nrincidencias, 58);
        }
        
        if($incideCSind == 'S'){
            array_push($nrincidencias, 32);
        }
        
        if($incideSalarioFamilia == 'S'){
            array_push($nrincidencias, 33);
        }
        
        if($incideDeducaoINSS == 'S'){
            array_push($nrincidencias, 224);
        }
        
        return $nrincidencias;
    }
    
    /**
     * Função que realiza a inclusão das incidências, de acordo com o número do tipo da incidência e os parâmetros passados. Caso a incidência exista, esta não será cadastrada novamente
     * @param array("NREVENTOM", "DTMESCOMPETENC", "NRORG", "IDINCIDEVENTO") $parametros   : Array que carrega informações importantes para a indentificação dos registros das incidências
     * 
     */
    public function salvaIncidencias($nrtiposincide, $parametros){
        $this->entityManager->beginTransaction();
        
        try {
            foreach($nrtiposincide as $nrtipoincide){
                $nreventom = $parametros['NREVENTOM'];
                $nrorg = $parametros['NRORG'] ?? $this->hcmParameters->getNrorgTrabalho();
                $incidevento = $parametros['IDINCIDEVENTO'];
                if ($this->verificaExistenciaRegistro($nrorg, $nrtipoincide, $nreventom, $incidevento)) {
                    continue;
                }
                
                $incidencia = new FpaIncidevento();
                $incidencia->setCdoperinclusao(array_key_exists('CDOPERINCLUSAO', $parametros) ? $parametros : $this->hcmParameters->getCdoperador());
                $incidencia->setDtinclusao(array_key_exists('DTINCLUSAO', $parametros) ? $parametros['DTINCLUSAO'] : DateUtil::getDataAtual());
                $incidencia->setDtmescompetenc(DateUtil::getDatadeString($parametros['DTMESCOMPETENC'],DateUtil::FORMATO_BRASILEIRO, true));
                $incidencia->setIdincidevento($incidevento);
                $incidencia->setNreventom($nreventom);
                $incidencia->setNrincidevento(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'FPA_INCIDEVENTO', 1, 12, true));
                $incidencia->setNrorg($nrorg);
                $incidencia->setNrorginclusao(array_key_exists('NRORGINCLUSAO', $parametros) ? $parametros['NRORGINCLUSAO'] : (array_key_exists('NRORG', $parametros) ? $parametros['NRORG'] : $this->hcmParameters->getNrorgTrabalho()));
                $incidencia->setNrtipoincide($nrtipoincide);
                $incidencia->setCdoperultatu(array_key_exists('CDOPERULTATU', $parametros) ? $parametros['CDOPERULTATU'] : (array_key_exists('CDOPERINCLUSAO', $parametros) ? $parametros : $this->hcmParameters->getCdoperador()));
                $incidencia->setNrorgultatu(array_key_exists('NRORGULTATU', $parametros) ? $parametros['NRORGULTATU'] : (array_key_exists('NRORG', $parametros) ? $parametros['NRORG'] : $this->hcmParameters->getNrorgTrabalho()));
                $incidencia->setDtultatu(array_key_exists('DTULTATU', $parametros) ? $parametros['DTULTATU'] : (array_key_exists('DTINCLUSAO', $parametros) ? $parametros['DTINCLUSAO'] : DateUtil::getDataAtual()));
                
                $this->entityManager->persist($incidencia);
            }
            $this->entityManager->flush();
            $this->entityManager->commit();
        } catch(\Exception $e){
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    /**
     * 
     * Função que realiza a exclusão das incidências, de acordo com o número do tipo da incidência e os parâmetros passados
     * @param array("NREVENTOM", "DTMESCOMPETENC", "NRORG", "IDINCIDEVENTO") $parametros   : Array que carrega informações importantes para a indentificação dos registros das incidências
     * 
     */
    public function deleteIncidencias($nrtipoincidencias, $parametros){
        try{
            $fpaincidevento = Repositories::FPA_INCIDEVENTO;
            $incidenciasExcluir = $this->entityManager->getRepository($fpaincidevento)->findBy(array('nreventom' => $parametros["NREVENTOM"], 'nrtipoincide' => $nrtipoincidencias, 'nrorg' => $parametros["NRORG"]));
            $this->entityManager->beginTransaction();
            if(is_array($incidenciasExcluir)){
                foreach($incidenciasExcluir as $incidencia){
                    $this->entityManager->remove($incidencia);
                }
                $this->entityManager->flush();
                $this->entityManager->commit();
            } else if(!empty($incidenciasExcluir)){
                $this->entityManager->remove($incidenciasExcluir);
                $this->entityManager->flush();
                $this->entityManager->commit();
            }
        } catch (\Exception $e){
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    /**
     * @param array("NREVENTOM", "DTMESCOMPETENC", "NRORG", "IDINCIDEVENTO") $parametros   : Array que carrega informações para a indentificação dos registros das incidências
     * @return Object array
     */
    public function buscaIncidencias($nrtipoincidencias, $nreventom, $nrorg){
        $fpaincidevento = Repositories::FPA_INCIDEVENTO;
        $incidencias = $this->entityManager->getRepository($fpaincidevento)->findBy(array('nreventom' => $nreventom, 'nrtipoincide' => $nrtipoincidencias, 'nrorg' => $nrorg));
        if(is_array($incidencias)){
            $nrIncidenciasCadastrados = [];
            foreach($incidencias as $incidencia){
                $nrIncidenciasCadastrados[] = $incidencia->getNrtipoincide();
            }
        } else if(!empty($incidencias)){
            $nrIncidenciasCadastrados[] = $incidencias->getNrtipoincide();
        }
        
        return $nrIncidenciasCadastrados ? $nrIncidenciasCadastrados : array();
    }
    
    /** 
     * 
     * Função que realiza a busca por incidências que ainda não foram cadastradas no evento, em caso de edição de um evento.
     * @param array("NREVENTOM", "DTMESCOMPETENC", "NRORG", "IDINCIDEVENTO") $parametros   : Array que carrega informações importantes para a indentificação dos registros das incidências
     * 
     */
    public function buscaIncidenciasAindaNaoCadastradas($nrincidencias, $parametros){
        $nrIncidenciasCadastrados = $this->buscaIncidencias($nrincidencias, $parametros['NREVENTOM'], $parametros['NRORG']);
        
        return array_diff($nrincidencias, $nrIncidenciasCadastrados);
    }
    
    /** 
     * Função que realiza a busca por incidências já foram cadastradas no evento, em caso de edição de um evento, e que devem ser excluídas.
     * @param array("NREVENTOM", "DTMESCOMPETENC", "NRORG", "IDINCIDEVENTO") $parametros   : Array que carrega informações importantes para a indentificação dos registros das incidências
     * 
     */
    public function buscaIncidenciasASeremExcluidas($nrincidencias, $parametros){
        return $nrIncidenciasCadastrados = $this->buscaIncidencias($nrincidencias, $parametros['NREVENTOM'], $parametros['NRORG']);
    }
    
    #Função que atualiza os historicos das incidencias para todas as competencias (somente checkbox que adicionem incidencias)
    public function atualizaHistoricosIncidencias($incideinss, $incidefgts, $incideirrf, $incide13, $incideferias, $incideCompFerias, $incideCompProv, $incidePeric, $incideInsalub, $incidePensAlim, $incidePensAlimFe, $incideDSRHoraExt, $incideDSRAdicNot, $incideCSind, $incideSalarioFamilia, $incideDeducaoINSS, $parametros, $estadoTiposIncidenciaEventoAnterior){
        $this->entityManager->beginTransaction();
        try {
            $nrorg = $this->hcmParameters->getNrorgTrabalho();
            $fpahistoricoevento = Repositories::FPA_EVENTOH;
            $historicos = $this->entityManager->getRepository($fpahistoricoevento)->findBy(array('nreventom' => $parametros["NREVENTOM"], 'nrorg' => $parametros["NRORG"]));
            
            foreach($historicos as $historico) {
                $historico->setNrincidcp($incideinss);
                $historico->setNrincidirrf($incideirrf);
                $historico->setNrincidfgts($incidefgts);
                $historico->setIdrepdecterc($incide13);
                $historico->setIdrepferias($incideferias);
                $historico->setIdcompsalfer($incideCompFerias);
                $historico->setIdcompsalprov($incideCompProv);
                $historico->setIdincideperic($incidePeric);
                $historico->setIdincideinsalub($incideInsalub);
                $historico->setIdincidepensalim($incidePensAlim);
                $historico->setIdincidepensalimfe($incidePensAlimFe);
                $historico->setIdincidedsrhoraext($incideDSRHoraExt);
                $historico->setIdincidedsradicnot($incideDSRAdicNot);
                $historico->setIdincidecsind($incideCSind);
                $historico->setIdsalariofamilia($incideSalarioFamilia);
                $historico->setIddeducaoinss($incideDeducaoINSS);
            }
            
            $this->entityManager->flush();
            $this->entityManager->commit();
        } catch(\Exception $e){
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
} 

