<?php

namespace HCM\Geral\Controllers;

use Doctrine\ORM\EntityManager;
use HCM\Geral\Logic\ImportacaoXmlEsocial;
use HCM\Util\DateUtil;
use HCM\Util\FileUploader;
use HCM\Util\FPA;
use HCM\Util\HCMParameters;
use HCM\Util\Repositories;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\HTTP\Kernel;

class MigracaoXmlController extends Crud {

    protected $dataSourceName = 'setMigracaoXml';
    protected $dataSourceNameGet = 'getMigracaoXml';
    protected $dataSourceNameGetEvento = 'getMigracaoXmlEvento';
    protected $dataSourceNameGetLog = 'getLogMigracaoXml';
    
    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO');
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, EntityManager $entityManager, HCMParameters $hcmParameters, Kernel $httpKernel, FPA $fpa){
        $this->parameterBag = $parameterBag;
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->httpKernel = $httpKernel;
        parent::__construct($dataSourceManager);
    }

    public function getMigracaoXml(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceNameGet);
            $filterData->addCondition('P_NRORG','=',$this->hcmParameters->getNrorgTrabalho());
            $filterData->addCondition('P_NRORG_PADRAO','=',$this->hcmParameters->getNrorgPadrao());
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData);
            $rows = $dataSet->getRows();
            $response->addDataSet(new DataSet($this->dataSourceNameGet, $rows));
        } catch(\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }
    
    public function setArquivosMigracaoXml(DTO\Request\Row $request, DTO\Response $response) {
        try {
            set_time_limit(0);
            
        
            $row = $request->getRow();
            
            foreach ($row['DSPATH'] as $dspath) {
                $xmlFile = $this->httpKernel->factoryFromFileData($dspath, $this->hcmParameters->getTempDirectory()."/");
                
                if (!$xmlFile->isFile()) {
                    throw new \Exception("Falha ao importar o arquivo");
                }
                
                $tmpFile = (DIRECTORY_SEPARATOR == "/" ? DIRECTORY_SEPARATOR: "") . $xmlFile->getPathname();
                $fileName = $dspath['name'];
                
                $logic = new ImportacaoXmlEsocial($this->hcmParameters->getNrorgTrabalho(), $this->hcmParameters->getNrorgPadrao(), $this->hcmParameters->getCdoperador(), $this->hcmParameters->getDtCompetencia());
                
                try {
                    $migracaoxml = $logic->setMigracaoXml($tmpFile, $fileName);
                } catch (\Exception $e) {
                    if ($e->getCode() == '10') {
                        $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
                    } else {
                        throw $e;
                    }
                }
            }
            
            $response->addDataSet(new DataSet('setArquivosMigracaoXml', array(true)));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteMigracaoXml(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $delete = true;
            
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (isset($row['NRMIGRACAOXML'])) {
                    $migracaoxml = $this->entityManager->getRepository(Repositories::ESO_MIGRACAOXML)->findOneBy(array('nrorg' => $this->hcmParameters->getNrorgTrabalho(), 'nrmigracaoxml' => $row['NRMIGRACAOXML']));
                    if($migracaoxml->getIdstatus() == 'C'){
                        return;
                    }
                    
                    //Apaga o arquivo do FileServer
                    FileUploader::deleteFile($migracaoxml->getDsarquivoxml());
                    
                    //Apaga o log de importação caso exista
                    /* RETIRADA EXCLUSAO DOS LOGS POIS A CONSTRAINT FOI ALTERADA PARA DELETE CASCADE
                    $logs = $this->entityManager->getRepository(Repositories::ESO_LOGMIGRACAOXML)->findBy(array('nrorg' => $this->hcmParameters->getNrorgTrabalho(), 'nrmigracaoxml' => $row['NRMIGRACAOXML']));
                    foreach ($logs as $logmigracaoxml) {
                        $this->entityManager->remove($logmigracaoxml);
                        $this->entityManager->flush();
                    }*/
                } else {
                    $arquivos = $this->entityManager->getRepository(Repositories::ESO_MIGRACAOXML)->findBy(array('nrorg' => $this->hcmParameters->getNrorgTrabalho(), 'nmevento' => $row['NMEVENTO'], 'idstatus' => ['P','A']));
                    foreach ($arquivos as $migracaoxml) {
                        if($migracaoxml->getIdstatus() == 'C'){
                            return;
                        }
                        
                        //Apaga o arquivo do FileServer
                        FileUploader::deleteFile($migracaoxml->getDsarquivoxml());
                        
                        //Apaga o log de importação caso exista
                        /* RETIRADA EXCLUSAO DOS LOGS POIS A CONSTRAINT FOI ALTERADA PARA DELETE CASCADE
                        $logs = $this->entityManager->getRepository(Repositories::ESO_LOGMIGRACAOXML)->findBy(array('nrorg' => $migracaoxml->getNrorg(), 'nrmigracaoxml' => $migracaoxml->getNrmigracaoxml()));
                        foreach ($logs as $logmigracaoxml) {
                            $this->entityManager->remove($logmigracaoxml);
                            $this->entityManager->flush();
                        }*/
                        
                        $this->entityManager->remove($migracaoxml);
                        $this->entityManager->flush();
                        
                        $response->addNotification(new DTO\Response\Notification('Delete success', DTO\Response\Notification::TYPE_SUCCESS));
                        $delete = false;
                    }
                }
            }
            
            if ($delete) {
                parent::delete($request, $response);
            }
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }    
    }

    public function getMigracaoXmlEvento(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceNameGetEvento);
            $filterData->addCondition('P_NRORG','=',$this->hcmParameters->getNrorgTrabalho());
            $filterData->addCondition('P_NRORG_PADRAO','=',$this->hcmParameters->getNrorgPadrao());
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData);
            $rows = $dataSet->getRows();
            $response->addDataSet(new DataSet($this->dataSourceNameGet, $rows));
        } catch(\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }

    public function getLogMigracaoXml(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceNameGetLog);
            $filterData->addCondition('P_NRORG','=',$this->hcmParameters->getNrorgTrabalho());
            $filterData->addCondition('P_NRORG_PADRAO','=',$this->hcmParameters->getNrorgPadrao());
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData);
            $rows = $dataSet->getRows();
            $response->addDataSet(new DataSet($this->dataSourceNameGet, $rows));
        } catch(\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }

    public function importarMigracaoXml(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $this->entityManager->beginTransaction();
            
            $row  = $request->getRow();
            
            $logic = new ImportacaoXmlEsocial($this->hcmParameters->getNrorgTrabalho(), $this->hcmParameters->getNrorgPadrao(), $this->hcmParameters->getCdoperador(), $this->hcmParameters->getDtCompetencia());

            if (is_null($row['NMREGISTROPAI'])) {
                // Faz importação por evento
                $nmevento = $row['NMEVENTO'];
                $logic->importarMigracaoXmlPorEvento($nmevento);
            } else {
                // Faz importação por arquivo
                $migracaoxml = $this->entityManager->getRepository(Repositories::ESO_MIGRACAOXML)->findOneBy(array('nrorg' => $this->hcmParameters->getNrorgTrabalho(), 'nrmigracaoxml' => $row['NRMIGRACAOXML']));
                $logic->importarMigracaoXml($migracaoxml);
            }
            
            $response->addDataSet(new DataSet('importarMigracaoXml', array($logic->getStatus())));
            if(!empty($logic->getMessages())){
                foreach($logic->getMessages() as $message) {
                    $response->addMessage(new DTO\Response\Message($message, DTO\Response\Message::TYPE_MESSAGE));
                }
            }
                
            $this->entityManager->commit();
            
        } catch(\Exception $e) {
            $this->entityManager->rollback();
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function agendarMigracaoXml(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row  = $request->getRow();
            
            $logic = new ImportacaoXmlEsocial($this->hcmParameters->getNrorgTrabalho(), $this->hcmParameters->getNrorgPadrao(), $this->hcmParameters->getCdoperador(), $this->hcmParameters->getDtCompetencia());
            
            if (is_null($row['NMREGISTROPAI'])) {
                // Faz importação por evento
                $nmevento = $row['NMEVENTO'];
        
                $paramsHcmParametros = $this->setParametrosHcm();
                
                $logic->agendarMigracaoXmlPorEvento($nmevento, $paramsHcmParametros);
            } else {
                // Faz importação por arquivo
                $migracaoxml = $this->entityManager->getRepository(Repositories::ESO_MIGRACAOXML)->findOneBy(array('nrorg' => $this->hcmParameters->getNrorgTrabalho(), 'nrmigracaoxml' => $row['NRMIGRACAOXML']));
                $logic->agendarMigracaoXml($migracaoxml);
            }
            
            $response->addDataSet(new DataSet('agendarMigracaoXml', array($logic->getStatus())));
            foreach($logic->getMessages() as $message) {
                $response->addMessage(new DTO\Response\Message($message, DTO\Response\Message::TYPE_MESSAGE));
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    private function setParametrosHcm(){
        return array(
            "cdoperador"        => $this->hcmParameters->getCdoperador(),
            "nmoperador"        => $this->hcmParameters->getNmoperador(),
            "nrestruttrab"      => $this->hcmParameters->getEstruturaTrabalho(),
            "nrorgpadrao"       => strval($this->hcmParameters->getNrorgpadrao()),
            "nrorg"             => strval($this->hcmParameters->getNrorg()),
            "nrorgtrab"         => strval($this->hcmParameters->getNrorgTrabalho()),
            "dtmescompetenc"    => $this->hcmParameters->getDtCompetencia(),
            "dtmescompetencdiacorrente" => $this->hcmParameters->getPrimeiroDiaDoMes(),
            "nrtipoestrutlegal" => $this->hcmParameters->getTipoEstruturaLegalOrg(),
            "nrtipoestrutgeren" => $this->hcmParameters->getTipoEstruturaGerencialOrg(),
            "idambientesal"     => $this->hcmParameters->getIdambientesal(),
            "nrocorrecal"       => $this->hcmParameters->getOcorrenciaCalculo(),
            "nrtpmodalidcal"    => $this->hcmParameters->getNrModalidadeCalculo(),
            "nrtpmodalidsal"    => $this->hcmParameters->getNrModalidadeSalario(),
            "dataatual"         => DateUtil::getDataAtual(),
            "dtultimodiames"    => $this->hcmParameters->getUltimoDiaMes(),
            "dtultimodiacompetencia" => $this->hcmParameters->getDtmescomptencDiaCorrente(),
        );
    }
    
    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
} 