<?php
namespace HCM\Util\ModeloImportacao;

use \Importacao\Logic\Integracao\Schema\Coluna;
use \Importacao\Logic\Integracao\Schema\Tabela;
use \Importacao\Logic\Integracao\GeradorSQL;
class QueryBuilder 
{
	private $data;
	private $helpers;

	public function __construct($data = array(), $helpers = array())
	{
		$this->data = $data;
		$this->helpers = $helpers;
	}

	public function build() 
	{
		$builtSql = "";
		$tabelas = $this->preparaTabelas();

		foreach ($tabelas as $tabela) {
			$geradorSql = new GeradorSQL();
			$geradorSql->setTabela($tabela);
			$builtSql .= $geradorSql->geraInsert();
		}

		return $builtSql;
	}

	private function preparaTabelas() 
	{
		$tabelas = array();
		foreach ($this->data as $entity) {
			if (!isset($tabelas[$entity->getTableName()])) {
				$tabela = new Tabela($entity->getTableName());
				foreach ($entity->getFields() as $fieldName => $fieldValue) {
					$tabela->addColuna(new Coluna($fieldName));
				}

				$tabelas[$entity->getTableName()] = $tabela;
			}

			$tabela = $tabelas[$entity->getTableName()];
			$tabela->addRow($entity->getFields());
		}

		return $tabelas;
	}

	public function buildGrupos($intervaloCommits = 200) 
	{
		$builtSql = "";
		$counter = 1;
		$primeiraEntidade = $this->data[0];
		foreach ($this->data as $entity) {		
			$geradorSql = new \Importacao\Logic\Integracao\GeradorSQL();
			$tabela     = new \Importacao\Logic\Integracao\Schema\Tabela($entity->getTableName());
			foreach ($entity->getFields() as $fieldName => $fieldValue) {
				$tabela->addColuna(new \Importacao\Logic\Integracao\Schema\Coluna($fieldName));
			}

			$tabela->addRow($entity->getFields());
			$geradorSql->setTabela($tabela);

			// inserir quebra de linha entre os grupos de inserts.
			if ($primeiraEntidade->getTableName() === $entity->getTableName()) {
				$builtSql .= "\n";
			}

			// nao gerar commit para cada registro.
			$builtSql .= $geradorSql->geraInsert(false);
			$builtSql .= $geradorSql->geraUpdate($this->helpers["primary_keys"][strtoupper($tabela->getNome())], false);


			// inserir commit para cada intervalo de linhas.
			if ($counter < count($this->data)) {
				$proximaEntidade = $this->data[$counter];
			}

			if (($counter >= $intervaloCommits) && 
				($proximaEntidade->getTableName() === $primeiraEntidade->getTableName())) {
				$builtSql .= \Importacao\Logic\Integracao\GeradorSQL::COMMIT."\n";
				$counter = 0;
			}
			$counter++;
		}

		$builtSql .= \Importacao\Logic\Integracao\GeradorSQL::COMMIT;

		return $builtSql;
	}
}