/******************************************************************************/
/* Sistemas Integrados TECNISA                                                */
/*                                                                            */
/* Compras e Suprimentos                                                      */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/* Nome: 8435_SUPSTP_VERIFICA_COTACAO                                         */
/*                                                                            */
/* Descrio:                                                                 */
/*  - Verifica se aquela cotao pode ou no ser marcada na hora da apurao  */
/*    dos melhores preos, levando en conta:                                  */
/*                                  o valor mnimo  de AF por fornecedor      */
/*                                  a quantidade mnima por fornecedor        */
/*                                  a quantidade mnima por produto/forncedor */
/*                                                                            */
/* Criao:                                                                   */
/*    06/08/2002 - Joo Fernando dos Reis Corra                              */
/*                                                                            */
/******************************************************************************/
IF EXISTS (SELECT * FROM SYSOBJECTS O, SYSUSERS U
          WHERE
            O.NAME = 'VERIFICA_COTACAO' AND
            TYPE = 'P'                      AND
            U.UID = O.UID                   AND
            U.NAME = USER)

   DROP PROCEDURE VERIFICA_COTACAO;
GO

CREATE PROCEDURE VERIFICA_COTACAO
(
   @P_CDFILIAL     VARCHAR(4),
   @P_CDFORNECED   VARCHAR(21),
   @P_CDPRODUTO    VARCHAR(10),
   @P_IDAPURPRXFOR VARCHAR(3),
   @P_VRPRECUNIAP  NUMERIC(15, 5),
   @P_QTACOMP      NUMERIC(10, 3),
   @P_MARCACOL     VARCHAR OUTPUT
)
AS
   DECLARE @V_QTMINAUTFOT NUMERIC(10, 3);
   DECLARE @V_VRMINAUTFOR NUMERIC(15, 2);
   DECLARE @V_QTMINAF     NUMERIC(10, 3);
BEGIN
   SET @P_MARCACOL = 'S';
   IF SUBSTRING(@P_IDAPURPRXFOR, 1, 1) = 'S' AND @P_MARCACOL = 'S' BEGIN
      SELECT @V_VRMINAUTFOR = ISNULL(MAX(VRMINAUTFOR), 0)
      FROM FORNECEDOR
      WHERE CDFORNECED = @P_CDFORNECED;
      IF @P_VRPRECUNIAP < @V_VRMINAUTFOR BEGIN
         SET @P_MARCACOL = 'N';
      END;
   END;
   IF SUBSTRING(@P_IDAPURPRXFOR, 2, 1) = 'S' AND @P_MARCACOL = 'S' BEGIN
      SELECT @V_QTMINAUTFOT = ISNULL(MAX(QTMINAUTFOT), 0)
      FROM FORNECEDOR
      WHERE CDFORNECED = @P_CDFORNECED;
      IF @P_QTACOMP < @V_QTMINAUTFOT BEGIN
         SET @P_MARCACOL = 'N';
      END;
   END;
   IF SUBSTRING(@P_IDAPURPRXFOR, 3, 1) = 'S' AND @P_MARCACOL = 'S' BEGIN
      SELECT @V_QTMINAF = ISNULL(MAX(QTMINAF), 0)
      FROM PRODFORN
      WHERE CDFORNECED = @P_CDFORNECED AND
            CDPRODUTO  = @P_CDPRODUTO;
      IF @P_QTACOMP < @V_QTMINAF BEGIN
         SET @P_MARCACOL = 'N';
      END;
   END;
END;
GO